import numpy as np
from glob import glob
from PIL import Image
from planetaryimage import PDS3Image
from skimage import exposure
import os

# create _geometry, and _texture directories if they do not exist
geometry_folder = "_geometry"
texture_folder = "_texture"
os.makedirs(geometry_folder, exist_ok=True)
os.makedirs(texture_folder, exist_ok=True)

# load all img files in images directory
img_files = glob("images/*.IMG")

# find minimum and maximum in images
img_max = 0
img_min = 0
first_image = True

for img_file in img_files:
    try:
        img = PDS3Image.open(img_file)
        
        if( first_image ):
            img_min = np.min(img.image)
            img_max = np.max(img.image)

        else:
            local_min = np.min(img.image)
            if( img_min > local_min ):
                img_min = local_min
            local_max = np.max(img.image)
            if( img_max < local_max ):
                img_max = local_max
                
        first_image = False

    except Exception as e:
        print(f"Error processing file {img_file}: {e}")
        
for img_file in img_files:
    try:
        img = PDS3Image.open(img_file)
        
        img_data = img.image
        
        #crop black edges
        img_data = img_data[4:1198, 25:1629]
        
        # apply CLAHE
        img_data_clahe = exposure.equalize_adapthist(img_data)
        
        # scale image data to 8-bit range (result of equalize_adapthist are float numbers in interval [0,1])
        img_data_clahe = img_data_clahe * 255 
        img_data_clahe = np.clip(img_data_clahe, 0, 255).astype(np.uint8)        

        # save CLAHE image as JPEG file in _geometry directory with same name as original file
        jpeg_file_geometry = os.path.join(geometry_folder, os.path.splitext(os.path.basename(img_file))[0] + ".jpg")
        Image.fromarray(img_data_clahe).save(jpeg_file_geometry, quality=100)
        

        # rescale image data to 8-bit range before gamma correction 
        img_data = (img_data - img_min) / (img_max - img_min) * 255
        img_data = np.clip(img_data, 0, 255).astype(np.uint8)        
        
        # gamma correction
        img_data_gamma = exposure.adjust_gamma(img_data, 0.65 )
        img_data_gamma = np.clip(img_data_gamma, 0, 255).astype(np.uint8)

        # save gamma corrected image as JPEG file in _texture directory with same name as original file
        jpeg_file_texture = os.path.join(texture_folder, os.path.splitext(os.path.basename(img_file))[0] + ".jpg")
        Image.fromarray(img_data_gamma).save(jpeg_file_texture, quality=100)

        print(f"Processed image saved in {jpeg_file_texture}")

    except Exception as e:
        print(f"Error processing file {img_file}: {e}")
